/**
 * En esta clase se implementan las funciones que realizan acciones de entrada/salida de los archivos:
 * 		- Cargar los archivos GTFS para poblar la ontología
 * 		- Guardar la ontología
 * 		- Guardar/actualizar archivo CSV con consultas SPARQL
 * 		- Guardar consulta en un archivo CSV
 * 
 * En esta clase a diferentes métodos que son estáticos. Esto se ha hecho asi para no tener que crear
 * instancias de la clase en los diferentes archivos, a la función tan solo hay que llamarla utilizando
 * el nombre de la clase y la función a utilizar (p.e.: HandleFiles.salidaArchivo(modeloFinal))
 * 
 * @author Adrián
 */
package UOC.AppPopulate_GTFS_PI;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.swing.JOptionPane;
import javax.swing.table.DefaultTableModel;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.jena.ontology.OntModel;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


public class HandleFiles {
	
	private static final Logger log = LogManager.getLogger(HandleFiles.class);
	
	public HandleFiles(){}
	
	/**
	 * En esta función se leen los datos de los archivos GTFS, que están en un formato CSV.
	 * 
	 * @param bufferedReader - Contiene un buffer con los datos obtenidos de los archivos 
	 * @return List<List<String>> - Se devuelve una lista doble con las filas leídas del archivo
	 */
	public List<List<String>> readFileFromBufferedReader(BufferedReader bufferedReader){
		List<List<String>> fileRows = new ArrayList<List<String>>();
		try{
			// Se obtiene la primera línea del archivo...
			String line = bufferedReader.readLine(); 
			// ...No se lee la primera fila del archivo (nombre de las columnas)
			boolean firstLine = true; 
			while (line != null && line.length() > 0) {
				if(firstLine == false){
					if (line.charAt(0) != '#') {
						List<String> rowValues = new ArrayList<String>();
						String[] tokens = line
								.split(",(?=([^\"]*\"[^\"]*\")*(?![^\"]*\"))");
						for (String token : tokens) {
							// Se eliminan espacios y comillas
							String processedToken = stripQutoes(token);
							rowValues.add(processedToken);
						}
						fileRows.add(rowValues);
					}
				}else{
					firstLine = false;
				}
				line = bufferedReader.readLine();
			}
			bufferedReader.close();
		}catch(FileNotFoundException e){
			log.error("Error: archivos GTFS no encontrados");
			e.printStackTrace();
		}catch(IOException e){
			log.error("Error cargando los archivos GTFS");
			e.printStackTrace();
		}
		return fileRows;
	}
	
	/**
	 * En esta función se eliminan los espacios del principio y final de la cadena.
	 * Además también se eliminan las comillas dobles que pueda haber al principio y al final.
	 * 
	 * @param token - Cadena leída del archivo y que hay que procesar
	 * @return String - Se devuelve la cadena procesada
	 */
	private String stripQutoes(String token) {
		// Se eliminan los espacios de los extremos
		String tokenWithoutSpaces = token.trim();
		// Se eliminan las comillas de los extremos
		if (tokenWithoutSpaces.length() > 0) {
			if (tokenWithoutSpaces.charAt(0) == '"'
					&& tokenWithoutSpaces
							.charAt(tokenWithoutSpaces.length() - 1) == '"') {
				return tokenWithoutSpaces.substring(1,
						tokenWithoutSpaces.length() - 1);
			}
		}
		return tokenWithoutSpaces;
	}
	
	/**
	 * Función utilizada para leer el archivo XML de puntos de interés. Se seleccinan
	 * los tags: name, web, body(descripción), latitude, longitude y address.
	 * 
	 * @param nameFile - Contiene el nombre del fichero XML que hay que analizar.
	 * @return List<List<String>> - Se devuelve una lista doble con las propiedades elegidas del archivo XML.
	 */
	public List<List<String>> leerXML(String nameFile){
		List<List<String>> fileRows = new ArrayList<List<String>>();
		try {
			// Cargamos el archivo con los puntos de interés
			File archivo = new File(getClass().getClassLoader().getResource(nameFile).toURI());
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder documentBuilder = dbf.newDocumentBuilder();
            Document document = documentBuilder.parse(archivo);
            document.getDocumentElement().normalize();
            
            // Seleccionamos el tag que tiene por nombre “service” y que engloba al resto de datos
            NodeList listaServicios = document.getElementsByTagName("service");
            
            // Recorremos la lista de nodos que acabamos de recuperar
            for (int temp = 0; temp < listaServicios.getLength(); temp++) {
                Node nodo = listaServicios.item(temp);
                
                /* Si el nodo es un elemento, seleccionamos los datos que
                 * vamos a utilizar y los insertamos en la lista */
                if (nodo.getNodeType() == Node.ELEMENT_NODE) {
                	List<String> rowValues = new ArrayList<String>();
                    Element element = (Element) nodo;
                    
                    rowValues.add(element.getElementsByTagName("name").item(0).getTextContent());
                    rowValues.add(element.getElementsByTagName("web").item(0).getTextContent());
                    rowValues.add(element.getElementsByTagName("body").item(0).getTextContent());
                    rowValues.add(element.getElementsByTagName("latitude").item(0).getTextContent());
                    rowValues.add(element.getElementsByTagName("longitude").item(0).getTextContent());
                    rowValues.add(element.getElementsByTagName("address").item(0).getTextContent());
                    
                    fileRows.add(rowValues);
                }
            }
        } catch (Exception e) {
        	log.error("Error al cargar el archivo XML");
            e.printStackTrace();
        }
		return fileRows;
	}
	
	/**
	 * Función utilizada para guardar la ontología una vez ha sido poblada.
	 * 
	 * @param modeloFinal - Contiene la OntModel poblada y que se tiene que guardar
	 */
	public static void salidaArchivo(OntModel modelofinal){
		try{
			String nameModelFile ="modelo";
			File rdfFile = new File(nameModelFile+".rdf");
			
			// Se guarda el modelo con extension *.rdf
			if (!rdfFile.exists()){
				rdfFile.createNewFile();
			}
			modelofinal.writeAll( new OutputStreamWriter( new FileOutputStream(rdfFile), "utf-8"),"RDF/XML");
			
			// Se guarda con extension .*ttl
			File ttlFile = new File(nameModelFile+".ttl");
			if (!ttlFile.exists()){
				ttlFile.createNewFile();
			}
			modelofinal.writeAll( new OutputStreamWriter( new FileOutputStream(ttlFile), "utf-8"),"TTL");
			
			// Se guarda con extension .*owl
			File owlFile = new File(nameModelFile+".owl");
			if(!owlFile.exists()){
				owlFile.createNewFile();
			}
			modelofinal.writeAll(new OutputStreamWriter(new FileOutputStream(owlFile), "utf-8"), "RDF/XML");
			//modelofinal.writeAll(new OutputStreamWriter(System.out, "utf-8"), "RDF/XML");
		}catch(FileNotFoundException e){
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Función utilizada para guardar una consulta SPARQL.
	 * El archivo de salida tendrá un formato CSV (consulta.csv).
	 * 
	 * @param nombre - Contiene el nombre asignado a la consulta.
	 * @param consulta - Contiene las instrucciones SPARQL a guardar.
	 */
	public static void guardarConsulta(String nombre, String consulta){
		// Guardamos la consulta en un archivo csv
		FileWriter fw = null;
		BufferedWriter bw = null;
		
		/* Si el nombre tiene una coma la remplazamos por una barra y
		 * en la consulta remplazamos el salto de línea por el símbolo
		 * hexadecimal de salto de línea */ 
		String datos = nombre.replace(",", "_") + "," + consulta.replace("\n", "\\u000A").replace(",", "%2C") +"\n"; 
		try{
			File archivo = new File("consulta.csv");
			if(!archivo.exists()){
				archivo.createNewFile();
			}
			
			// flag true, indica adjuntar informacion al archivo
			fw = new FileWriter(archivo.getAbsoluteFile(), true);
			bw = new BufferedWriter(fw);
			bw.write(datos);
		}catch(IOException e){
			e.printStackTrace();
		}finally{
			// Liberamos los recursos 
			try{
				if(bw != null)
					bw.close();
				if(fw != null)
					fw.close();
			}catch(IOException e){
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Función utilizada para eliminar una consulta SPARQL y sobrescribir el archivo de consultas.
	 * El archivo de salida tendrá un formato CSV (consulta.csv).
	 * 
	 * @param consultas - Contiene un HashMap donde las claves son el nombre de la consulta y los valores son las instrucciones SPARQL. 
	 */
	public static void guardarConsulta(HashMap<String, String> consultas){
		FileWriter fw = null;
		BufferedWriter bw = null;
		
		try{
			// Abrimos el archivo de consultas
			File archivo = new File("consulta.csv");
			if(!archivo.exists()){
				archivo.createNewFile();
			}
			// flag false, sobrescribir el archivo
			fw = new FileWriter(archivo.getAbsoluteFile(), false);
			bw = new BufferedWriter(fw);
			
			// Escribimos en el archivo las consultas que tenemos guardadas en el HashMap
			Iterator it = consultas.keySet().iterator();
			while(it.hasNext()){
				String key = (String)it.next();
				String datos = key + "," + consultas.get(key).replace("\n", "\\u000A") +"\n";
				//System.out.println("datos: "+datos);
				bw.write(datos);
			}	
		}catch(IOException e){
			e.printStackTrace();
		}finally{
			// Liberamos los recursos 
			try{
				if(bw != null)
					bw.close();
				if(fw != null)
					fw.close();
			}catch(IOException e){
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Función utilizada para recuperar las consultas SPARQL del archivo "consulta.csv".
	 * 
	 * @return HashMap<String, String> - Se devuelve un HashMap donde la clave es el nombre de la consulta y los datos son las instrucciones
	 */
	public static HashMap<String, String> recuperarConsultas(){
		FileReader fr = null;
		BufferedReader br = null;
		HashMap<String, String> consultas = new HashMap<String, String>();
		try{
			// Abrimos el archivo de consultas
			File archivo = new File("consulta.csv");
			if(!archivo.exists()){
				log.error("Archivo consulta.csv no existe todavia");
			}else{
				fr = new FileReader (archivo);
				br = new BufferedReader(fr);
				String linea="";
				
				// Recorremos el archivo leyendo las líneas y guardando los datos en un HashMap
				while((linea=br.readLine())!=null){
					String[] datos = linea.split(",");
					consultas.put(datos[0], datos[1]);
				}
			}
		}catch(IOException e){
			e.printStackTrace();
		}finally{
			// Liberamos los recursos
			try{
				if(br != null)
					br.close();
				if(fr != null)
					fr.close();
			}catch(IOException e){
				e.printStackTrace();
			}
		}
		return consultas;
	}
	
	/**
	 * Esta función guarda los datos de una consulta SPARQL, y que
	 * han sido mostrados en forma de tabla, en un archivo CSV. La
	 * primera fila serán los nombres de las columnas y el resto
	 * serán los datos.
	 * 
	 * @param path - Ruta donde guardar el archivo
	 * @param modelTabla - Los datos a guardar. Están en un DefaultTablemodel, que ha sido utilizado para mostrarlos en forma de tabla 
	 */
	public static void guardarArchivoCSV(String path, DefaultTableModel modeloTabla){
		// Abrimos/creamos el archivo seleccionado por el usuario 
		File archivo = new File(path);
		try{
			FileWriter escribir = new FileWriter(archivo, false);
			
			String cadena="";
			// Guardamos el nombre de las columnas en la primera fila
			for(int i=0;i<modeloTabla.getColumnCount();i++){
				cadena += modeloTabla.getColumnName(i) + ",";
			}
			cadena = cadena.substring(0, cadena.length()-1)+"\n";
			
			// Guardamos los datos, que serán el resto de filas
			for(int row=0;row<modeloTabla.getRowCount();row++){
				for(int column=0; column<modeloTabla.getColumnCount(); column++){
					cadena += modeloTabla.getValueAt(row, column)+",";
				}
				cadena = cadena.substring(0, cadena.length()-1)+"\n";
			}
			
			escribir.write(cadena);
			escribir.close();
		}catch(FileNotFoundException e){
			e.printStackTrace();
		}catch(IOException e){
			e.printStackTrace();
		}
	}

}
