/**
 * En esta clase se carga la ontología en la aplicación y se puebla con los datos contenidos
 * en los diferentes archivos GTFS y de puntos de interés.  
 * Lo primero que hacemos es cargar la ontología del archivo OWL que hemos creado con Protégé.
 * Una vez la tenemos cargada cargamos los datos procedentes de los archivo del estándar GTFS.
 * Para cargar estos datos hay que tener en cuenta que se tiene que hacer en un orden determinado
 * ya que hay campos de un archivo que están relacionados con los de otro archivo (todo y ser
 * archivos CSV están relacionados como una base de datos relacional). Por último se cargará el
 * archivo de datos de los puntos de interés. Por lo tanto el orden de carga de los archivos de
 * datos será:
 * 			- Archivo “agency.txt”: Contiene la información de la agencia que ofrece los servicios.
 * 			- Archivo “routes.txt”: Contiene los datos de las rutas proporcionadas por la agencia
 * 									que hemos cargado anteriormente.
 * 			- Archivo “calendar.txt”: Contiene los días de la semana en los que se ofrece un servicio.
 * 			- Archivo “trips.txt”: Contiene información sobre los viajes que contiene una ruta. En este
 * 								   archivo se relacionan la ruta, el calendario y los viajes.
 * 			- Archivo “stops.txt”: Contiene información sobre las paradas que tienen los diferentes viajes.
 * 			- Archivo “stop_times.txt”: Contiene información sobre la hora de llegada y salida de un
 * 										vehículo a una parada. En este archivo se relacionan los viajes
 * 										con las paradas.
 * 			- Archivo “turismo_v1_es.xml”: Este archivo contiene la información sobre los puntos
 * 										   de interés de la ciudad. Este archivo tiene un formato XML
 * 										   y por eso se carga el último, ya que se tiene que leer de
 * 										   una forma distinta a los anteriores.
 * 
 * @author Adrián
 */
package UOC.AppPopulate_GTFS_PI;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JOptionPane;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.jena.datatypes.xsd.XSDDatatype;
import org.apache.jena.geosparql.configuration.GeoSPARQLConfig;
import org.apache.jena.geosparql.spatial.SpatialIndex;
import org.apache.jena.geosparql.spatial.SpatialIndexException;
import org.apache.jena.ontology.Individual;
import org.apache.jena.ontology.OntClass;
import org.apache.jena.ontology.OntModel;
import org.apache.jena.ontology.OntModelSpec;
import org.apache.jena.query.Dataset;
import org.apache.jena.query.ParameterizedSparqlString;
import org.apache.jena.query.QueryExecution;
import org.apache.jena.query.QueryExecutionFactory;
import org.apache.jena.query.QueryParseException;
import org.apache.jena.query.QuerySolution;
import org.apache.jena.query.ResultSet;
import org.apache.jena.query.ResultSetFormatter;
import org.apache.jena.rdf.model.Literal;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.rdf.model.ModelFactory;
import org.apache.jena.rdf.model.ModelMaker;
import org.apache.jena.rdf.model.Property;
import org.apache.jena.rdf.model.RDFWriter;
import org.apache.jena.rdf.model.Statement;
import org.apache.jena.riot.RDFDataMgr;
import org.apache.jena.shared.BadURIException;
import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.core.config.Configurator;
import org.apache.logging.log4j.core.config.DefaultConfiguration;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


public class Populate {
	
	/* Log para imprimir por consola */
	private static final Logger log = LogManager.getLogger(Populate.class);
	
	/* Constantes */
	public static final String NSOPENGTFS = "http://www.GTFS-PI.org/GTFS-PI#";
	public static final String NSgeo = "http://www.w3.org/2003/01/geo/wgs84_pos#";
	
	/* Variables globales */
	OntModel om;
	public OntModel modelofinal;
	
	/* Data Properties de la ontologia*/
	private Property hasColor, hasDescription, hasDirection, hasName, hasType, hasURL, isAccessible, hasAddress, hasHeadsign,
			 latitude, longitude, arrivalTime, departureTime, monday, tuesday, wednesday, thursday, friday, saturday, sunday,
			 hasSequence;
	
	/* Object properties de la ontologia*/
	private Property geometry, hasAgency, hasCalendar, hasPlatform, hasRoute, hasStation, hasStop, isAgencyOf,
			 isCalendarOf, isPlatformOf, isRouteOf, isStationOf, isStopOf;

	/**
	 * Constructor de la clase.Constructor de la clase. Se carga la
	 * ontología desde el archivo y se inicializan las propiedades
	 * utilizadas.
	 * */
	public Populate(){
		Configurator.initialize(new DefaultConfiguration());
		Configurator.setRootLevel(Level.INFO);
		
		/* Cargamos el modelo desde el archivo*/
		log.info("Cargamos el modelo");
		Model m = RDFDataMgr.loadModel("file:src/main/java/data/GTFS-PI.owl");
		this.om = createOntModel(m);
		this.om.setNsPrefix("spatial", "http://jena.apache.org/spatial#");
		
		/* Se recuperan los Data properties de la ontologia cargada*/
		this.hasColor = this.om.getDatatypeProperty(NSOPENGTFS+"has_color");
		this.hasDescription = this.om.getDatatypeProperty(NSOPENGTFS+"has_description");
		this.hasDirection = this.om.getDatatypeProperty(NSOPENGTFS+"has_direction");
		this.hasAddress = this.om.getDatatypeProperty(NSOPENGTFS+"has_address");
		this.hasHeadsign = this.om.getDatatypeProperty(NSOPENGTFS+"has_headsign");
		this.hasName = this.om.getDatatypeProperty(NSOPENGTFS+"has_name");
		this.hasType = this.om.getDatatypeProperty(NSOPENGTFS+"has_type");
		this.hasURL = this.om.getDatatypeProperty(NSOPENGTFS+"has_url");
		this.isAccessible = this.om.getDatatypeProperty(NSOPENGTFS+"isAccessible");
		this.latitude = this.om.getDatatypeProperty(NSgeo+"lat");
		this.longitude = this.om.getDatatypeProperty(NSgeo+"long");
		this.arrivalTime = this.om.getDatatypeProperty(NSOPENGTFS+"arrival_time");
		this.departureTime = this.om.getDatatypeProperty(NSOPENGTFS+"departure_time");
		this.monday = this.om.getDatatypeProperty(NSOPENGTFS+"monday");
		this.tuesday = this.om.getDatatypeProperty(NSOPENGTFS+"tuesday");
		this.wednesday = this.om.getDatatypeProperty(NSOPENGTFS+"wednesday");
		this.thursday = this.om.getDatatypeProperty(NSOPENGTFS+"thursday");
		this.friday = this.om.getDatatypeProperty(NSOPENGTFS+"friday");
		this.saturday = this.om.getDatatypeProperty(NSOPENGTFS+"saturday");
		this.sunday = this.om.getDatatypeProperty(NSOPENGTFS+"sunday");
		this.hasSequence = this.om.getDatatypeProperty(NSOPENGTFS+"has_sequence");
		
		/* Se recuperan los Object Properties de la ontologia cargada */
		this.geometry = this.om.getProperty(NSgeo + "geometry");
		this.hasAgency = this.om.getProperty(NSOPENGTFS + "hasAgency");
		this.hasCalendar = this.om.getProperty(NSOPENGTFS + "hasCalendar");
		this.hasPlatform = this.om.getProperty(NSOPENGTFS + "hasPlatform");
		this.hasStation = this.om.getProperty(NSOPENGTFS + "hasStation");
		this.hasRoute = this.om.getProperty(NSOPENGTFS + "hasRoute");
		this.hasStop = this.om.getProperty(NSOPENGTFS + "hasStop");
		this.isAgencyOf = this.om.getProperty(NSOPENGTFS + "isAgencyOf");
		this.isCalendarOf = this.om.getProperty(NSOPENGTFS + "isCalendarOf");
		this.isPlatformOf = this.om.getProperty(NSOPENGTFS + "isPlatformOf");
		this.isRouteOf = this.om.getProperty(NSOPENGTFS + "isRouteOf");
		this.isStationOf = this.om.getProperty(NSOPENGTFS + "isStationOf");
		this.isStopOf = this.om.getProperty(NSOPENGTFS + "isStopOf");
	}
	
	/**
	 * En esta función se leen los datos de los archivos GTFS y 
	 * de los puntos de interés. Una vez se han leído se insertan
	 * en la ontología   
	 */
	public void configuration(){
		/* Leemos los archivos GTFS */
		List<List<String>> fileContents = null;
		InputStream is = null;
		
		// Inicializamos todas las funciones de filtrado y propiedades GeoSPARQL
		GeoSPARQLConfig.setupMemoryIndex();
		
		System.out.println("========================================");
		System.out.println("== Leemos los archivos e introducimos ==");
		System.out.println("==       los datos en el modelo       ==");
		System.out.println("========================================");

		HandleFiles handleFile = new HandleFiles();
		
		// Leemos los archivos GTFS y cargamos los datos en el modelo
		for(int i=0; i<6;i++){
			try {
				switch(i){
					case 0: log.info("Cargamos el archivo agency");
							is = getClass().getClassLoader().getResourceAsStream("data/agency.txt");
						break;
					case 1:	log.info("Cargamos el archivo routes");
							is = getClass().getClassLoader().getResourceAsStream("data/routes.txt");
						break;
					case 2:	log.info("Cargamos el archivo calendar");
							is = getClass().getClassLoader().getResourceAsStream("data/calendar.txt");
						break;
					case 3:	log.info("cargamos el archivo trips");
							is = getClass().getClassLoader().getResourceAsStream("data/trips.txt");
						break;
					case 4:	log.info("Cargamos el archivo stops");
							is = getClass().getClassLoader().getResourceAsStream("data/stops.txt");
						break;
					case 5:	log.info("Cargamos el archivo stops_times");
							is = getClass().getClassLoader().getResourceAsStream("data/stop_times.txt");
						break;
				}
				
				// Abrimos el archivo correspondiente para procesar los datos
				InputStreamReader inputStreamReader = new InputStreamReader(is);
				BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
				fileContents = handleFile.readFileFromBufferedReader(bufferedReader);
				
				bufferedReader.close();
				inputStreamReader.close();
			} catch (FileNotFoundException e) {
				log.error("File not found");
				e.printStackTrace();
			} catch (IOException e) {
				log.error("I/O error");
				e.printStackTrace();
			}
			if (fileContents == null) {
				return;
			}
			
			// Nos dirigimos al método correspondiente para procesar los datos del archivo
			switch(i){
				case 0: om = crearAgencia(fileContents, om);
					break;
				case 1: om = crearRoute(fileContents, om);
					break;
				case 2: om = crearCalendar(fileContents, om);
					break;
				case 3: om = crearTrip(fileContents, om);
					break;
				case 4: om = crearStops(fileContents, om);
					break;
				case 5: om = crearStopsTimes(fileContents, om);
					break;
				default:
			}
		}
		
		// Cargamos y procesamos el archivo con los datos de los puntos de interés
		log.info("Cargamos el archivo turismo_v1_es.xml");
		fileContents = handleFile.leerXML("data/turismo_v1_es.xml");
		om = crearPuntosInteres(fileContents, om);
		
		this.modelofinal = om;
		
		System.out.println("========================================");
		System.out.println("========================================");
	}
	
	/**
	 * En esta función se crean las agencias con la información
	 * que se les puede asociar.
	 * 
	 * @param fileContents - Contiene una lista con las agencias y su información
	 * @param m - Contiene el modelo con los datos cargados anteriormente
	 * @return OntModel - Devuelve un OntModel con las agencias añadidas
	 */
	private OntModel crearAgencia(List<List<String>> fileContents, OntModel m){
		Individual i;
		for(List<String> row : fileContents){
			// Creamos una agencia
			OntClass agency_id = m.getOntClass(NSOPENGTFS + "Agency");
			i = agency_id.createIndividual(NSOPENGTFS+row.get(0));
			
			// Añadimos las data properties del archivo 
			i.addProperty(this.hasName, row.get(1));
			i.addProperty(this.hasURL, row.get(2));
		}
		return m;
	}
	
	/**
	 * En esta función se crean las rutas que ofrece una agencia.
	 * 
	 * @param fileContents - Contiene una lista con las rutas y su información
	 * @param m - Contiene el modelo con los datos cargados anteriormente
	 * @return OntModel - Devuelve un OntModel con las rutas añadidas
	 */
	private OntModel crearRoute(List<List<String>> fileContents, OntModel m){
		Individual i;
		for(List<String> row : fileContents){
			// Creamos una ruta
			OntClass route_id = m.getOntClass(NSOPENGTFS + "Route");
			i = route_id.createIndividual(NSOPENGTFS + row.get(0));
			
			// Añadimos los data properties
			i.addProperty(this.hasName, row.get(3));
			i.addProperty(this.hasURL, row.get(6));
			i.addProperty(this.hasColor, row.get(7));
			switch(row.get(5)){
				case "0": 	// Tranvia
							i.addProperty(this.hasType, "tranvia");
					break;
				case "1": 	// subterraneo o metro
							i.addProperty(this.hasType, "metro");
					break;
				case "2": 	// tren
							i.addProperty(this.hasType, "tren");
					break;
				case "3": 	// autobus
							i.addProperty(this.hasType, "autobus");
					break;
				case "4": 	//Transbordador
							i.addProperty(this.hasType, "transbordador");
					break;
				case "5": 	// Tranvia de cable
							i.addProperty(this.hasType, "tranvia de cable");
					break;
				case "6": 	//teleferico
							i.addProperty(this.hasType, "teleferico");
					break;
				case "7": 	//Funicular
							i.addProperty(this.hasType, "funicular");
					break;
				case "11": 	//Trolebus
							i.addProperty(this.hasType, "trolebus");
					break;
				case "12": 	//Monorriel
							i.addProperty(this.hasType, "monorriel");
					break;
			}
			
			// Añadimos las propiedades del objeto
			i.addProperty(this.hasAgency, row.get(1));
			m.getIndividual(NSOPENGTFS+row.get(1)).addProperty(this.isAgencyOf, row.get(0));
		}
		return m;
	}
	
	/**
	 * En esta función añadimos la informacion sobre los dias
	 * de la semana en que está disponible un servicio.
	 * 
	 * @param fileContents - Contiene una lista de los servicios y su información
	 * @param m - Contiene el modelo con los datos cargados anteriormente
	 * @return OntModel - Devuelve un OntModel con la disponibilidad de los servicios añadidos
	 */
	private OntModel crearCalendar(List<List<String>> fileContents, OntModel m){
		Individual i;
		for(List<String> row : fileContents){
			// Creamos un calendario de los servicios
			OntClass calendar_id = m.getOntClass(NSOPENGTFS + "Calendar");
			i = calendar_id.createIndividual(NSOPENGTFS + row.get(0));
			
			// Añadimos las propiedades
			i.addProperty(this.monday, row.get(1), XSDDatatype.XSDboolean);
			i.addProperty(this.tuesday, row.get(2), XSDDatatype.XSDboolean);
			i.addProperty(this.wednesday, row.get(3), XSDDatatype.XSDboolean);
			i.addProperty(this.thursday, row.get(4), XSDDatatype.XSDboolean);
			i.addProperty(this.friday, row.get(5), XSDDatatype.XSDboolean);
			i.addProperty(this.saturday, row.get(6), XSDDatatype.XSDboolean);
			i.addProperty(this.sunday, row.get(7), XSDDatatype.XSDboolean);
		}
		return m;
	}
	
	/**
	 * En esta funcion se añaden los viajes que hay en una ruta y
	 * se relaciona con los dias que están disponibles.
	 * 
	 * @param fileContents - Contiene una lista con los viajes y su información
	 * @param m - Contiene el modelo con los datos cargados anteriormente
	 * @return OntModel - Devuelve un OntModel con los viajes añadidos
	 */
	private OntModel crearTrip(List<List<String>> fileContents, OntModel m){
		Individual i;
		for(List<String> row : fileContents){
			// Creamos un viaje
			OntClass trips_id = m.getOntClass(NSOPENGTFS + "Trips");
			i = trips_id.createIndividual(NSOPENGTFS + row.get(2));
			
			// Añadimos las propiedades
			i.addProperty(this.hasHeadsign, row.get(3));
			i.addProperty(this.hasName, row.get(4));
			i.addProperty(this.hasDirection, row.get(5), XSDDatatype.XSDint);
			//i.addProperty(this.isAccessible, row.get(8));
			esAccesible(i, row.get(8));
			
			// Añadimos las propiedades del objeto
			i.addProperty(this.hasRoute, row.get(0));
			m.getIndividual(NSOPENGTFS+row.get(0)).addProperty(this.isRouteOf, row.get(2));
			i.addProperty(this.hasCalendar, row.get(1));
			m.getIndividual(NSOPENGTFS+row.get(1)).addProperty(this.isCalendarOf, row.get(2));
		}
		return m;
	}
	
	/**
	 * Función que añade las parades que tiene un viaje. Se relacionan las diferentes paradas con su padre:
	 * 		- Estaciones: no tienen padres
	 * 		- Paradas, entradas/salidas o nodos genéricos: tienen como padre a una estación
	 * 		- Áreas de abordaje: tienen como padre a una parada
	 * 
	 * @param fileContents - Contiene una lista con las paradas y su información
	 * @param m - Contiene el modelo con los datos cargados anteriormente
	 * @return OntModel - Devuelve un OntModel con las paradas añadidas
	 */
	private OntModel crearStops(List<List<String>> fileContents, OntModel m){
		Individual i;
		/* Tenemos que insertar los elementos en orden ya que unos dependen de otros:
		 * 		- Estaciones
		 * 		- Paradas, entradas/salidas, nodos genericos
		 * 		- Areas de abordaje
		 */
		for(List<String> row : fileContents){
			// Primero insertamos las estaciones
			if(row.get(8).equals("1")){
				// Creamos una estación
				OntClass stop_id = m.getOntClass(NSOPENGTFS + "Station");
				i = stop_id.createIndividual(NSOPENGTFS + row.get(0));
				
				// Añadimos las propiedades
				i = propiedadesTrip(i, row);
				addLatLong(row.get(4),row.get(5),m,i);
			}
		}
		for(List<String> row : fileContents){
			// Añadimos las paradas, entradas/salidas, Nodo generico
			if(row.get(8).equals("0") || row.get(8).equals("2") || row.get(8).equals("3")){
				OntClass stop_id;
				if(row.get(8).equals("0")){
					// Creamos una parada
					stop_id = m.getOntClass(NSOPENGTFS + "Stop");
				}
				else if(row.get(8).equals("2")){
					// Creamos una entrada/salida
					stop_id = m.getOntClass(NSOPENGTFS + "Entrance_exit");
				}
				else{
					// Creamos un nodo genérico
					stop_id = m.getOntClass(NSOPENGTFS + "Generic_node");
				}
				
				i = stop_id.createIndividual(NSOPENGTFS + row.get(0));
				
				// Añadimos las propiedades
				i = propiedadesTrip(i, row);
				addLatLong(row.get(4),row.get(5),m,i);
				
				// Añadimos las propiedades del objeto
				if(!row.get(9).equals("")){
					i.addProperty(this.hasStation, row.get(9));
					m.getIndividual(NSOPENGTFS+row.get(9)).addProperty(this.isStationOf, row.get(0));
				}
			}
		}
		for(List<String> row : fileContents){
			// Añadimos las Areas de abordaje
			if(row.get(8).equals("4")){
				// Creamos una Área de abordaje
				OntClass stop_id = m.getOntClass(NSOPENGTFS + "Boarding_area");
				i = stop_id.createIndividual(NSOPENGTFS + row.get(0));
				
				i = propiedadesTrip(i, row);
				addLatLong(row.get(4),row.get(5),m,i);
				
				// Añadimos las propiedades del objeto
				i.addProperty(this.hasPlatform, row.get(9));
				m.getIndividual(NSOPENGTFS+row.get(9)).addProperty(this.isPlatformOf, row.get(0));
			}
		}
		return m;
	}
	
	/**
	 * Función con los Data Properties que hay que añadir a los viajes
	 * 
	 * @param i - Individuo al cual hay que añadir las propiedades
	 * @param row - Lista con las propiedades a añadir
	 * @return Individual - Devolvemos al individuo con las propiedades añadidas
	 */
	private Individual propiedadesTrip(Individual i, List<String> row){
		// Añadimos las propiedades
		i.addProperty(this.hasName, row.get(2));
		i.addProperty(this.hasDescription, row.get(3));
		i.addProperty(this.hasURL, row.get(7));
		//i.addProperty(this.isAccessible, row.get(11));
		esAccesible(i, row.get(11));
		
		return i;
	}
	/**
	 * Función que inserta si una parada o un viaje és
	 * accesible en silla de ruedas
	 * @param i - Individuo al cual hay que añadir las propiedades
	 * @param access - Cadena con la información de acceso
	 */
	private void esAccesible(Individual i, String access){
		switch(Integer.parseInt(access)){
			case 0: i.addProperty(this.isAccessible, "Sin informacion");
				break;
			case 1: i.addProperty(this.isAccessible, "És accesible");
				break;
			case 2: i.addProperty(this.isAccessible, "No és accesible");
				break;
		}
	}
	/**
	 * En esta función se relacionan los viajes con las paradas que tiene,
	 * además se añaden la hora de llegada y salida de los vehículos.
	 * 
	 * @param fileContents - Contiene una lista donde se relacionan los Stops con los Trips
	 * @param m - Contiene el modelo con los datos cargados anteriormente
	 * @return OntModel - Devuelve un OntModel con los Trips y los Stops relacionados
	 */
	private OntModel crearStopsTimes(List<List<String>> fileContents, OntModel m){
		Individual i;
		for(List<String> row : fileContents){
			/* Recurperamos al individuo Stop_type y le añadimos
			 * las propiedades de hora de llegada y de salida */
			i = m.getIndividual(NSOPENGTFS + row.get(3));
			i.addProperty(this.arrivalTime, row.get(1), XSDDatatype.XSDtime);
			i.addProperty(this.departureTime, row.get(2), XSDDatatype.XSDtime);
			i.addProperty(this.hasSequence, row.get(4), XSDDatatype.XSDint);
			
			// Añadimos las propiedades del objeto
			i.addProperty(this.isStopOf, row.get(0));
			m.getIndividual(NSOPENGTFS+row.get(0)).addProperty(this.hasStop, row.get(3));
		}
		return m;
	}
	
	/**
	 * En esta función se inserta la información sobre los puntos de interés en la ontología.
	 * 
	 * @param fileContents - Contiene una lista con la información de los puntos de interés
	 * @param m - Contiene el modelo con los datos cargados anteriormente
	 * @return OntModel - Devuelve un OntModel con los puntos de interés añadidos
	 */
	private OntModel crearPuntosInteres(List<List<String>> fileContents, OntModel m){
		Individual i;
		for(List<String> row : fileContents){
			OntClass interestPoint_id = m.getOntClass(NSOPENGTFS + "Interest_point");
			
			i = interestPoint_id.createIndividual(NSOPENGTFS + cambiarCaracteres(row.get(0)).replace(" ","-").trim());
			
			// Añadimos las propiedades
			i.addProperty(this.hasName, cambiarCaracteres(row.get(0)));
			i.addProperty(this.hasURL, row.get(1));
			i.addProperty(this.hasDescription, eliminarTags(cambiarCaracteres(row.get(2))).trim());
			i.addProperty(this.hasAddress, cambiarCaracteres(row.get(5)));
			addLatLong(row.get(3), row.get(4), m, i);
		}
		return m;
	}
	
	/**
	 * En esta función se cambian los caracteres HTML que se utilizan
	 * en el archivo XML de los puntos de interés.
	 * 
	 * @param cadena - Contiene la cadena a la cual hay que cambiar los caracteres
	 * @return String - Se devuelve la cadena con los caracteres cambiados.
	 */
	private String cambiarCaracteres(String cadena){
		cadena = cadena.replace("&aacute;", "á");
		cadena = cadena.replace("&Aacute;", "Á");
		cadena = cadena.replace("&eacute;", "é");
		cadena = cadena.replace("&Eacute;", "É");
		cadena = cadena.replace("&iacute;", "í");
		cadena = cadena.replace("&Iacute;", "Í");
		cadena = cadena.replace("&oacute;", "ó");
		cadena = cadena.replace("&Oacute;", "Ó");
		cadena = cadena.replace("&uacute;", "ú");
		cadena = cadena.replace("&Uacute;", "Ú");
		cadena = cadena.replace("&ntilde;", "ñ");
		cadena = cadena.replace("&Ntilde;", "Ñ");
		cadena = cadena.replace("&euro;", "€");
		cadena = cadena.replace("&nbsp;", " ");
		return cadena;
	}
	
	/**
	 * Funció utilizada para eliminar los Tags HTML utilizados en la
	 * descripción de los puntos de interés.
	 * 
	 * @param cadena - Cadena en la que hay que eliminar los Tags
	 * @return String - Se devuelve la cadena sin los tags 
	 */
	public String eliminarTags(String cadena){
		while(true){
			int leftTag = cadena.indexOf("<");
			if(leftTag < 0){ return cadena; }
			int rightTag = cadena.indexOf(">");
			if(rightTag < 0){ return cadena; }
			cadena = cadena.substring(0, leftTag) + " " + cadena.substring(rightTag+1);
		}
	}
	
	
	/**
	 * Se genera un modelo OntModel a partir del modelo cargado desde el archivo OWL
	 * 
	 * @param model - Contiene el modelo que se ha cargado desde el archivo OWL
	 * @return OntModel - Se devuelve un ontModel
	 */
	private OntModel createOntModel(Model model) {
		ModelMaker maker = ModelFactory.createMemModelMaker();
		OntModelSpec spec = new OntModelSpec(OntModelSpec.OWL_MEM_MICRO_RULE_INF);
		spec.setImportModelMaker(maker);
		OntModel m = ModelFactory.createOntologyModel(spec, model);
		return m;
	}
	
	/**
	 * Función utilizada para añadir las propiedades de latitud
	 * y longitud al individuo seleccionado
	 * 
	 * @param latitud - Variable con la latitud a añadir
	 * @param longitud - Variable con la longitud a añadir
	 * @param m - Modelo con el que se está trabajando
	 * @param i - Individuo al cual hay que añadir las coordenadas
	 */
	public void addLatLong(String latitud, String longitud, OntModel m, Individual i){
		/*OntClass punto = m.getOntClass(NSgeo + "point");
		punto.createIndividual(NSgeo+"Point_"+longitud+"_"+latitud);*/
		
		Literal lat = m.createTypedLiteral(latitud, XSDDatatype.XSDdouble);
		Literal lon = m.createTypedLiteral(longitud, XSDDatatype.XSDdouble);
		Statement statmLat = m.createStatement(i, this.latitude, lat);
		Statement statmLong = m.createStatement(i, this.longitude, lon);
		
		//HAY QUE PONER UN POINT
		Statement statmGeometry = m.createStatement(i, this.geometry, lon+"_"+lat);
		m.add(statmLat);
		m.add(statmLong);
		m.add(statmGeometry);
	}
	
	/******************** CONSULTAS SPARQL Y GEOSPARQL ******************************************/
	
	/**
	 * En esta función utilizamos la consulta SPARQL introducida por
	 * el usuario y el modelo para obtener los datos que coincidentes.
	 * 
	 * @param query - Contiene la consulta SPARQL
	 * @return ResultSet - Contiene los resultados obtenidos después de ejecutar la consulta sobre el modelo
	 */
	public ResultSet resultadoSPARQL(String query) throws Exception{
		log.info("query: "+query.replace("\n", " "));
		
		ResultSet results = this.executeQuery(query, this.modelofinal);
		
		return results;
	}
	
	/**
	 * En esta función ejecutamos la consulta SPARQL sobre el modelo de
	 * la ontología que ha sido poblada. Si no sucede ningún error se
	 * devuelven los resultados obtenidos, sino se muestra un mensaje con
	 * el error producido.
	 * 
	 * @param queryString - Contiene la consulta SPARQL que hay que ejecutar
	 * @param om - Contiene el modelo poblado con la información de los archivos
	 * @return ResultSet - Devolvemos los resultados obtenidos
	 */
	public ResultSet executeQuery(String queryString, Model om) {
		QueryExecution exec = null;
		try{
			// Creamos una cadena parametrizada en la que se puede inyectar valores
			ParameterizedSparqlString pss = new ParameterizedSparqlString();
			
			/*Creamos un objeto Spatial Index que nos permitirá comprobar
			 * colisiones entre geometrías de una forma eficaz.También lo
			 * necesitamos para poder utilizar la información geoespacial */
			Dataset dataset = SpatialIndex.wrapModel(om);
			pss.setCommandText(queryString);
			//log.info("--------\n"+pss.toString()+"\n-------");
			
			// Ejecutamos la consulta sobre el modelo
			exec =  QueryExecutionFactory.create(pss.asQuery(),dataset);
			
			return exec.execSelect();
		}catch(QueryParseException e){
			/* Si ha ocurrido un error con la consulta (variable mal escrita,
			 * falta alguna llave,…) se muestra un error indicándolo. */ 
			log.error("Consulta SPARQL incorrecta. Linea: "+e.getLine()+", Columna: "+e.getColumn()+", "+e.getMessage());
			JOptionPane.showMessageDialog(null, "Error en la consulta SPARQL. \n Línea: "+e.getLine()+", Columna: "+e.getColumn()+"\n "+e.getMessage(),
					"Error consulta", JOptionPane.ERROR_MESSAGE);
			//e.printStackTrace();
		} catch (SpatialIndexException e) {
			/* Si ocurre un error con el índice espacial también es muestra
			 * un mensaje indicándolo. */
			log.error("Error en el índice espacial. "+e.getMessage());
			JOptionPane.showMessageDialog(null, "Error en el índice espacial. \n"+e.getMessage(), "Error SpatialIndex", JOptionPane.ERROR_MESSAGE);
			//e.printStackTrace();
		}

		// Si se produce un error devolvemos null
		return null;

	}
}
