/**
 * En esta clase se implementa el JFrame que mostrará el mapa con
 * los lugares que tienen información geoespacial. Esta ventana
 * contendrá en la parte izquierda un árbol con todos los lugares
 * disponibles y en la parte derecha el mapa con el cual se podrá
 * interactuar. 
 * Parte del código contenido en esta clase ha sido extraído de los
 * tutoriales para desarrolladores proporcionados por OpenStreetMap.
 * 
 * @author Adrián
 */
package UOC.appInterface;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;

import org.apache.jena.query.QuerySolution;
import org.apache.jena.query.ResultSet;
import org.apache.jena.rdf.model.Literal;
import org.openstreetmap.gui.jmapviewer.Coordinate;
import org.openstreetmap.gui.jmapviewer.JMapViewer;
import org.openstreetmap.gui.jmapviewer.JMapViewerTree;
import org.openstreetmap.gui.jmapviewer.Layer;
import org.openstreetmap.gui.jmapviewer.LayerGroup;
import org.openstreetmap.gui.jmapviewer.MapMarkerDot;
import org.openstreetmap.gui.jmapviewer.OsmTileLoader;
import org.openstreetmap.gui.jmapviewer.events.JMVCommandEvent;
import org.openstreetmap.gui.jmapviewer.interfaces.JMapViewerEventListener;
import org.openstreetmap.gui.jmapviewer.interfaces.MapMarker;
import org.openstreetmap.gui.jmapviewer.tilesources.OsmTileSource;

public class OsmMapViewer extends JFrame implements JMapViewerEventListener{

	private JMapViewerTree treeMap;
    private JLabel zoomLabel;
    private JLabel zoomValue;
    private JLabel mperpLabelName;
    private JLabel mperpLabelValue;

	/**
	 * Constructor de la clase OsmMapViewer
	 * 
	 * @param results Contiene el resultado de la consulta realizada a la ontologia con todos los lugares con coordenadas
	 */
	public OsmMapViewer(ResultSet results) {
		treeMap = new JMapViewerTree("Lugares");
        
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setBounds(100, 100, 500, 500);
		setLayout(new BorderLayout());
		
		setupPanels();
		setupMap(results);
	}
	
	/**
	 * Esta función se encarga de configurar el mapa para añadir todos los lugares,
	 * tanto paradas como puntos de interés. También se encarga de controlar las
	 * acciones del ratón.
	 * 
	 * @param results Contiene el resultado de la consulta realizada a la ontologia con todos los lugares con coordenadas
	 */
	private void setupMap(ResultSet results){
		// Añadimos un Listener para manejar las operaciones del usuario con el mapa
        map().addJMVListener(this);
        
        // Establecemos el tipo de mosaico y decimos que los marcadores son visibles
        map().setTileSource(new OsmTileSource.Mapnik());
        map().setTileLoader(new OsmTileLoader(map()));
        map().setMapMarkerVisible(true);
        map().setZoomContolsVisible(true);
        
        map().addMouseListener(new MouseAdapter(){
        	private String nombre="";
        	MapMarker currentMarker = null;
        	
        	/*
        	 * En esta función del evento, miramos que dot ha sido seleccionado
        	 * y cambiamos el nombre para mostrar las coordenadas 
        	 */
        	@Override
        	public void mousePressed(MouseEvent e) {
        		//System.out.println("-->"+e.getButton());
        		Point mousePoint = e.getPoint();
                for (MapMarker marker : map().getMapMarkerList()) {
                    Point p = map().getMapPosition(marker.getLat(), marker.getLon(), true);
                    if (p != null) {
                        int r = map().getRadius(marker, p);
                        Rectangle rect = new Rectangle(p.x - r, p.y - r, r + r, r + r);
                        if (rect.contains(mousePoint)) {
                            currentMarker = marker;
                            break;
                        }
                    }           
                }
        		
        		if (currentMarker instanceof MapMarkerDot) {
        				nombre = ((MapMarkerDot)currentMarker).getName();
        				((MapMarkerDot)currentMarker).setName("Lat: "+((MapMarkerDot)currentMarker).getLat()+
        										", Long: "+((MapMarkerDot)currentMarker).getLon());
        		}
                map().repaint();
        	}
        	
        	/*
        	 * En esta función del evento, cuando soltamos el botón del
        	 * ratón se vuelve a mostrar el nombre del lugar
        	 */
        	@Override
        	public void  mouseReleased(MouseEvent e) {
        		if(currentMarker != null){
        			((MapMarkerDot)currentMarker).setName(nombre);
        			nombre="";
        			currentMarker = null;
        			map().repaint();
                }
        	}
        });
        
        // Establecemos el zoom y el punto inicial donde se mostrará el mapa
        Coordinate c = new Coordinate(40.4669, -3.68917);
        map().setDisplayPosition(c, 17);
        
        // Obtenemos las columnas de la consulta
        List<String> columnas = results.getResultVars();
        
        // Añadimos dos grupos al árbol del mapa para separar entre paradas y puntos de interés
        LayerGroup interestPoints = new LayerGroup("Puntos de interés");
		LayerGroup Stops = new LayerGroup("Stops");
		
        for( ; results.hasNext() ; ){
			QuerySolution row = results.nextSolution();
			
			// Recuperamos las coordenadas de los lugares
			Literal lit = row.get(columnas.get(1)).asLiteral();

			// Seleccionamos tan solo el valor de la latitud y longitud
			String [] coord = lit.getString().split("http://www.w3.org/2001/XMLSchema#double");
			double lon = Double.parseDouble(coord[0].substring(0, coord[0].length()-2));
			double lat = Double.parseDouble(coord[1].substring(1, coord[1].length()-2));
			
			// Recuperamos el nombre del lugar para añadirlo al dot
			String name = row.get("name").toString();
			Layer layer;
			MapMarkerDot m;
			/* Miramos si és una parada o un lugar de interés.
			 * Dependiendo de qué tipo sea el dot, tendrá un color u otro. 
			 */
			if(row.get("classe").toString().contains("Stops")){
				layer = Stops.addLayer(name);
				m = new MapMarkerDot(layer, name, lat, lon);
				m.setBackColor(new Color(255,255,0));
			}else{
				layer = interestPoints.addLayer(name);
				m = new MapMarkerDot(layer, name, lat, lon);
				m.setBackColor(new Color(255,0,0));
			}
			// Añadimos el dot al mapa y al árbol
			map().addMapMarker(m);
			treeMap.addLayer(layer);
		}

        /*treeMap.setTreeVisible(true);
        add(treeMap, BorderLayout.CENTER);*/
        
        // Añadimos el mapa y el árbol de opciones a un splitPane
        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        splitPane.setOneTouchExpandable(true);
        splitPane.setDividerLocation(150);
        
        JScrollPane scrollPane1 = new JScrollPane();
        scrollPane1.setViewportView(treeMap.getTree());
        
        splitPane.setLeftComponent(scrollPane1);
        splitPane.setRightComponent(map());
        
        // Cuando un nodo del árbol cambia, redibujamos el mapa 
        treeMap.getTree().getModel().addTreeModelListener(new TreeModelListener() {
            @Override
            public void treeNodesChanged(final TreeModelEvent e) {
                repaint();
            }

            @Override
            public void treeNodesInserted(TreeModelEvent arg0) { }

            @Override
            public void treeNodesRemoved(TreeModelEvent arg0) { }

            @Override
            public void treeStructureChanged(TreeModelEvent arg0) { }
        });
        
        add(splitPane, BorderLayout.CENTER);
	}
	
	/**
	 * En esta función creamos los paneles que serán utilizados en esta ventana,
	 * asi como los mensajes de ayuda.
	 */
    private void setupPanels() {
        JPanel panel = new JPanel(new BorderLayout());
        JPanel panelTop = new JPanel();
        JPanel panelBottom = new JPanel();
        JPanel helpPanel = new JPanel();

        mperpLabelName = new JLabel("Metros/Pixels: ");
        mperpLabelValue = new JLabel(String.format("%s", map().getMeterPerPixel()));
        zoomLabel = new JLabel("Zoom: ");
        zoomValue = new JLabel(String.format("%s", map().getZoom()));

        add(panel, BorderLayout.NORTH);
        add(helpPanel, BorderLayout.SOUTH);
        panel.add(panelTop, BorderLayout.NORTH);
        panel.add(panelBottom, BorderLayout.SOUTH);
        JLabel helpLabel = new JLabel("Utilice el botón derecho del ratón para moverse,\n "
                + "o haga doble click con el izquierdo o utiliza la rueda para hacer zoom. Click en un punto para ver sus coordenadas");
        helpPanel.add(helpLabel);

        panelTop.add(zoomLabel);
        panelTop.add(zoomValue);
        panelTop.add(mperpLabelName);
        panelTop.add(mperpLabelValue);
    }
	
    /**
     * Esta función se encarga de devolver un viewer del mapa,
     * el cuál será utilizado para añadir los elementos
     * 
     * @return JMapViewer Se devuelve un visor del mapa
     */
	private JMapViewer map() {
        return treeMap.getViewer();
    }
	
	/**
	 * Esta función se encarga de actualizar el valor del zoom en los mensajes de ayuda
	 */
	private void updateZoomParameters() {
        if (mperpLabelValue != null)
            mperpLabelValue.setText(String.format("%s", map().getMeterPerPixel()));
        if (zoomValue != null)
            zoomValue.setText(String.format("%s", map().getZoom()));
    }

	/**
	 * Función que se encarga de procesar los comandos utilizados en el mapa (zoom y movimiento)
	 * 
	 * @param command Contiene el comando que ha iniciado el evento
	 */
	@Override
	public void processCommand(JMVCommandEvent command) {
		if (command.getCommand().equals(JMVCommandEvent.COMMAND.ZOOM) ||
                command.getCommand().equals(JMVCommandEvent.COMMAND.MOVE)) {
            updateZoomParameters();
        }
	}

}
