/**
 * En esta clase se implementa el JDialog que mostrará los resultados de
 * una consulta SPARQL en forma de tabla. El JDialog está dividido en dos
 * partes, la primera es donde está la tabla y la segunda los botones con
 * las acciones. La acción que se podrá realizar en esta ventana es exportar
 * los datos a un archivo CSV.
 * 
 * @author Adrián
 */
package UOC.appInterface;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Shape;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.table.DefaultTableModel;

import org.apache.jena.query.QuerySolution;
import org.apache.jena.query.ResultSet;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import UOC.AppPopulate_GTFS_PI.HandleFiles;

import javax.swing.JTable;
import javax.swing.JScrollPane;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.geom.RoundRectangle2D;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.awt.event.ActionEvent;

public class SolutionTable extends JDialog {

	private static final Logger log = LogManager.getLogger(SolutionTable.class);
	private final JPanel contentPanel = new JPanel();
	private JTable table;
	int x, y;

	/**
	 * Constructor de la clase. Se crea el JDialog con los datos obtenidos
	 * de la consulta SPARQL. Esta ventana consta con una tabla donde el
	 * nombre de las columnas son los nombres buscados con el SELECT de la
	 * consulta y las diferentes filas son los datos recuperados. También
	 * contiene una parte donde hay tres botones: uno para maximizar, otro
	 * para exportar los datos a un archivo CSV y el último para cerrar la ventana.
	 * 
	 * @param parent - Contiene el identificador de la ventana principal
	 * @param model - Indicamos si la ventana es modal
	 * @param results - Contiene los resultados obtenidos al realizar la consulta
	 */
	public SolutionTable(Frame parent, boolean model, ResultSet results) {
		super(parent, model);
		
		// Modificamos la apariencia de la ventana
		this.setUndecorated(true);
		setBackground(new Color(0, 0, 0, 0));
		setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		addComponentListener(new ComponentAdapter(){
			public void componentResized(ComponentEvent e){
				// Redondeamos los bordes de la ventana 
				Shape forma = new RoundRectangle2D.Double(0, 0, getBounds().width, getBounds().height, 27, 27);
				setShape(forma);
			}
		});
		
		this.setLocationRelativeTo(null);
		
		setBounds(560, 100, 450, 300);
		getContentPane().setLayout(new BorderLayout());
		
		// Creamos el panel que contendrá todos los elementos de la ventana
		JPanel panelVentana = new JPanel();
		panelVentana.setBorder(new EmptyBorder(10, 10, 10, 10));
		getContentPane().add(panelVentana, BorderLayout.CENTER);
		panelVentana.setBackground(new Color(0, 0, 0, 128));
		panelVentana.setLayout(new BorderLayout(0, 0));
		
		/************* contentPanel **************/
		
		//Creamos el panel que contiene la tabla de resultados
		contentPanel.setBackground(Color.LIGHT_GRAY);
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		panelVentana.add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(new BorderLayout(0, 0));
		
		/* Añadimos los Listener que permiten mover la ventana.
		 * Estos los tenemos que poner tanto en el contentPane1 como en el buttonPane.
		 */
		contentPanel.addMouseMotionListener(new MouseMotionAdapter() {
			@Override
			public void mouseDragged(MouseEvent evt) {
				// Actualizamos la posición de la ventana
				setLocation(getLocation().x + evt.getX() - x, getLocation().y + evt.getY() - y);
			}
		});
		contentPanel.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent evt) {
				// Obtenemos la posición X e Y de donde se a presionado el ratón
				x = evt.getX();
				y = evt.getY();
			}
		});
		
		// Recuperamos el nombre de las variables buscadas y creamos la tabla
		List<String> columnas = results.getResultVars();
		table = new JTable();
		DefaultTableModel modeloTabla = (DefaultTableModel)table.getModel();
		
		// Añadimos los nombres de las columnas
		for(int i=0;i<columnas.size();i++){
			modeloTabla.addColumn(columnas.get(i));
		}
		
		/* Creamos un objeto donde iremos añadiendo los datos recuperados
		 * de la consulta y que después introduciremos en la tabla */
		Object [] fila = new Object[columnas.size()];
		for( ; results.hasNext() ; ){
			QuerySolution row = results.nextSolution();
			for(int i=0;i<columnas.size();i++){
				fila[i] = row.get(columnas.get(i));
			}
			modeloTabla.addRow(fila);
		}
		// La tabla estará dentro de un JScrollPane para poder desplazarnos
		JScrollPane scrollPane = new JScrollPane(table);
		contentPanel.add(scrollPane);
		
		/*************  buttonPane  **************/
		
		// Añadimos el panel que contiene los botones
		JPanel buttonPane = new JPanel();
		buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
		
		// Añadimos los Listener que permiten mover la ventana igual que en contentPane1.
		buttonPane.addMouseMotionListener(new MouseMotionAdapter() {
			@Override
			public void mouseDragged(MouseEvent evt) {
				// Actualizamos la posición de la ventana
				setLocation(getLocation().x + evt.getX() - x, getLocation().y + evt.getY() - y);
			}
		});
		buttonPane.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent evt) {
				// Obtenemos la posición X e Y de donde se a presionado el ratón
				x = evt.getX();
				y = evt.getY();
			}
		});

		panelVentana.add(buttonPane, BorderLayout.SOUTH);
		
		// Botón de aceptar. Tan solo cerramos la ventana
		JButton okButton = new JButton("OK");
		okButton.setBackground(new Color(240,240,240));
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				dispose();
			}
		});
		
		// Botón para maximizar la ventana
		JButton btnMaximizar = new JButton("");
		btnMaximizar.setBackground(new Color(240,240,240));
		ImageIcon img2 = new ImageIcon(MainWindow.class.getResource("/resources/maximizar2.png"));
		btnMaximizar.setIcon(new ImageIcon(img2.getImage().getScaledInstance(25, 17, Image.SCALE_DEFAULT)));
		
		// Creamos el Listener que permiten maximizar la ventana y que vuelva a su estado original
		btnMaximizar.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				/* Si la ventana no está maximizada la maximizamos y quitamos los bordes redondeados.
				 * Por otro lado, si esta maximizada la devolvemos a su tamaño original */
				if((getBounds().width != GraphicsEnvironment.getLocalGraphicsEnvironment().getMaximumWindowBounds().width)&&
						((getBounds().height != GraphicsEnvironment.getLocalGraphicsEnvironment().getMaximumWindowBounds().height))){
					setBounds(GraphicsEnvironment.getLocalGraphicsEnvironment().getMaximumWindowBounds());
					panelVentana.setBorder(new EmptyBorder(0, 0, 0, 0));
					addComponentListener(new ComponentAdapter(){
						public void componentResized(ComponentEvent e){
							Shape forma = new RoundRectangle2D.Double(0, 0, getBounds().width, getBounds().height, 0, 0);
							setShape(forma);
						}
					});
				}else{
					setBounds(560, 100, 450, 300);
					panelVentana.setBorder(new EmptyBorder(10, 10, 10, 10));
					addComponentListener(new ComponentAdapter(){
						public void componentResized(ComponentEvent e){
							Shape forma = new RoundRectangle2D.Double(0, 0, getBounds().width, getBounds().height, 27, 27);
							setShape(forma);
						}
					});
				}
			}
		});
		buttonPane.add(btnMaximizar);
		
		// Botón para exportar los datos a un archivo CSV
		JButton btnCsv = new JButton("CSV");
		btnCsv.setBackground(new Color(240,240,240));
		
		/* Creamos un Listener el cual mostrara un dialogo que nos permitirá
		 * elegir el nombre y la ubicación del archivo que vamos a guardar */
		btnCsv.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				log.info("Guardamos la consulta en CSV");
				
				// Mostramos el dialogo para seleccionar el nombre y la ubicación
				JFileChooser guardar = new JFileChooser();
				guardar.setAcceptAllFileFilterUsed(false);
				guardar.setFileFilter(new FileNameExtensionFilter("CSV files (*csv)","csv"));
				int resultado = guardar.showSaveDialog(null);
				guardar.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
				
				// Si aceptamos, recuperamos la dirección seleccionada y le agregamos la extensión “.csv”
				if(resultado == guardar.APPROVE_OPTION){
					String path =  guardar.getSelectedFile().getAbsolutePath();
					if(!path.toLowerCase().endsWith(".csv")){
						path += ".csv";
					}
					// Guardamos el archivo pasando la ruta y los datos
					HandleFiles.guardarArchivoCSV(path, modeloTabla);
				}
			}
		});
		
		buttonPane.add(btnCsv);
		okButton.setActionCommand("OK");
		buttonPane.add(okButton);
		getRootPane().setDefaultButton(okButton);
	}

}
